//========================================================================
// Author: Keith Richie (krichie@krichie.com)
//
// DISCLAIMER: The sample code described herein is provided on an as is basis, 
// without warranty of any kind, to the fullest extent permitted by law. 
// Author does not warrant or guarantee the individual success developers 
// may have in implementing the sample code on their development platforms or 
// in using their own server configurations. 
//
// Author does not warrant, guarantee or make any representations regarding 
// the use, results of use, accuracy, timeliness or completeness of any data or 
// information relating to the sample code. Author disclaims all warranties, 
// express or implied, and in particular, disclaims all warranties of merchantability, 
// fitness for a particular purpose, and warranties related to the code, or any 
// service or software related thereto. 
//
// Author shall not be liable for any direct, indirect or consequential 
// damages or costs of any type arising out of any action taken by you or others 
// related to the sample code.
//
//========================================================================
using System;
using System.Text;
using Microsoft.SharePoint;
using System.IO;

namespace SPIEFolder
{
    class Program
    {

        static void Main(string[] args)
        {
            if (args.Length < 3)
            {

                string sFormat = "{0,5}{1,-20}{2}\n";
                StringBuilder sb = new StringBuilder();
                sb.Append("\nSYNTAX: SPIEFolder.exe <WebURL> <ListName> <Directory> [<import|export>]\r\n");
                sb.Append("\nwhere:\n\n");
                sb.AppendFormat(sFormat, " ", "<WebURL>", "The Web URL where"
                    + " the list exists");
                sb.AppendFormat(sFormat, " ", "<ListName>", "Name of the"
                    + " list (Document Library)");
                sb.AppendFormat(sFormat, " ", "<Directory>", "File System directory");
                sb.AppendFormat(sFormat, " ", "[<import|export>]", "Import or export"
                    + " (defaults to export)");
                Console.WriteLine(sb.ToString());
                return;
            }

            bool bImport = false;
            string strDirectory;

            SPSite site = null;
            SPWeb web = null;
            SPList list = null;
            try
            {
                site = new SPSite(args[0]);
                web = site.OpenWeb();

		web.Lists.IncludeRootFolder = true;

                list = web.Lists[args[1]];
                strDirectory = args[2].TrimEnd('\\');
                if (args.Length >= 4)
                {
                    if (args[3].ToLower().CompareTo("import") == 0)
                        bImport = true;
                }



                if (bImport)
                    ImportFromFileSystemFolder(list.RootFolder, strDirectory);
                else
                    ExportToFileSystemFolder(list.RootFolder, strDirectory);


            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }
            finally
            {


                if (web != null)
                    	web.Dispose();
                if (site != null)
                    site.Dispose();
            }
        }

        /// <summary>
        /// Allthough this simply calls ExtractFolder at this time, the reason I'm shelling this 
        /// is that I have plans to modify ExportToFileSystemFolder for additional features that will
        /// be beyond just what ExtractFolder does
        /// </summary>
        /// <param name="folder">The SPFolder to extract files from</param>
        /// <param name="strExportDir">The File System Path (Directory) to put the files in</param>
        private static void ExportToFileSystemFolder(SPFolder folder, string strExportDir)
        {
            ExtractFolder(folder, strExportDir);
        }
        
        /// <summary>
        /// Extracts files contained in a SPFolder and places them on the file system recursively
        /// Creates nested folders as necassary to mimic the SPFolder folder heirarchy
        /// </summary>
        /// <param name="folder">The SPFolder to extract files from</param>
        /// <param name="strExportPath">The File System Path (Directory) to put the files</param>
        /// <returns></returns>
        private static long ExtractFolder(SPFolder folder, string strExportPath)
		{

			Console.WriteLine("Processing Folder " + folder.ServerRelativeUrl);
			string strPath = null;
			long lFolderSize = 0;

			// If we're exporting the files from the web to the local file system, 
			//  create the appropriate file system folder
            if (strExportPath != null)
			{
                strPath = CheckPathAndCreate(strExportPath,folder);
			}

			foreach(SPFile file in folder.Files)
			{

					string sFileLoc = strPath + "\\" + file.Name;
					Console.WriteLine("    Exporting file " + file.Name + " to " + sFileLoc);
					byte[] binFile = file.OpenBinary();
					FileStream fs = new FileStream(sFileLoc,FileMode.OpenOrCreate,FileAccess.Write);
					fs.Write(binFile,0,binFile.Length);
					fs.Close();            

			}
			
				foreach(SPFolder subfolder in folder.SubFolders)
                    ExtractFolder(subfolder, strExportPath);
			return lFolderSize;
		}

		// Break apart the URL and make a File System directory heirarchy from it if needed.
        /// <summary>
        /// Creates a folder at strExportPath if it doesn't exist
        /// </summary>
        /// <param name="strExportPath">The File System Path (Directory) create the folder in</param>
        /// <param name="folder">The SPFolder for which we are creating the file system folder for.</param>
        /// <returns></returns>
		private static string CheckPathAndCreate(string strExportPath, SPFolder folder)
		{
			string strPath;

            strPath = strExportPath + '\\' + folder.Url.Replace("/", "\\");
			// ServerRelativeUrl will begin with "/" so we'll be cool to just append it onto m_strExportDir
			System.IO.Directory.CreateDirectory(strPath);
            // TODO: Add security code to mimic SPFolder security to File System

			return strPath;
		}

        /// <summary>
        /// Imports files from the file system recursively, and places them into a document library
        /// Creates nested SPFolder folders as necassary to mimic the file system folder heirarchy
        /// </summary>
        /// <param name="folder">The SPFolder to place the files in</param>
        /// <param name="strImportPath">The File System Path (Directory) to get files from</param>
        /// <returns></returns>
        private static void ImportFromFileSystemFolder(SPFolder folder, string strImportPath)
        {

            // Import files from location, then create new folders if necassary and import those
            ImportFilesToFolder(folder, strImportPath);
            DirectoryInfo dir = new DirectoryInfo(strImportPath);

            foreach (DirectoryInfo subdir in dir.GetDirectories())
            {
                Console.WriteLine("Creating folder for " + subdir.Name);
                folder.SubFolders.Add(subdir.Name);
                SPFolder subfolder = folder.SubFolders[subdir.Name];
                // TODO: Add security code to mimic File System security to SPFolder 

                ImportFromFileSystemFolder(subfolder, subdir.FullName);
            }
        }

        /// <summary>
        /// Imports files from a specific path into a specific SPFolder
        /// </summary>
        /// <param name="folder">The SPFolder to place the files in</param>
        /// <param name="strImportPath">The File System Path (Directory) to get files from</param>
        private static void ImportFilesToFolder(SPFolder folder, string strImportPath)
        {

            string msg = string.Empty;
            string strFileMask = "*.*";


            Console.WriteLine("Adding files to " + folder.Url + " from " + strImportPath);
            DirectoryInfo dir = new DirectoryInfo(strImportPath);

            foreach (FileInfo f in dir.GetFiles(strFileMask))
            {
                try
                {

                    byte[] buffer = new byte[f.Length];
                    System.IO.FileStream fs = new FileStream(f.FullName, System.IO.FileMode.Open);
                    fs.Read(buffer, 0, (int)f.Length);
                    fs.Close();
                    Console.WriteLine("Adding document " + f.Name);
                    folder.Files.Add(f.Name, buffer);
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.Message);
                }
            }
        }
    }
}
